import numpy as np
import matplotlib.pyplot as plt
from matplotlib import rc

rc("text", usetex = True)
R_earth = 6371
h_ion = 350
d = 10
#sample comment

def map3(el):
	f = R_earth/(R_earth + h_ion)
	el = np.deg2rad(el)
	map_function = 1/np.sqrt(1 - (f*np.cos(el))**2)
	return map_function

def map2(el):
	R = R_earth + h_ion
	p = 90 - el
	z = np.deg2rad(p)
	map_function = 1/np.cos(z) + (np.cos(z)**2 - 1)*d**2/(8*R**2*np.cos(z)**5)
	return map_function

def map1(el):
	el = np.deg2rad(el)
	map_function = 1/np.sin(el)
	return map_function
	
def map4(el):
	a0 = 64.4297
	a1 = 0.0942437
	a2 = 1.39436
	a3 = 19.6357
	b0 = 64.3659
	b1 = 0.104974
	b2 = 1.41152
	b3 = -0.0463341
	z = 90 - el
	z_rad = np.deg2rad(z)
	za = (z - a0)*a1
	za = np.deg2rad(za)
	zb = (z - b0)*b1
	zb = np.deg2rad(zb)
	a = (-np.arctan(za) - a2)*a3
	b = (-np.arctan(zb) - b2)*b3
	p = a + b*d
	map_function = 1/((1 - p/100)*np.cos(z_rad))
	return map_function



import numpy as np
import pandas as pd
import glob
import re
import itertools
import os

pattern1 = r'[0-9]+'
pattern = r'[0-9]{3}'

def day(files):
	"""Combines hourly information on TEC into blocks of information corresponding to one day
	
	Args:
		files (list): List of strings of file names
		
	Returns:
		dict: A dictionary whose keys are day and year, and values are Pandas dataframes corresponding to information from that day and year.
	"""
	titles = ['week', 'TOW', 'SVID', 'azimuth', 'elevation', 'TEC', 'locktime']
	years = []	#list of years in the files
	all_dfs = {}	#dictionary with all dataframes 
	all_year_days = {}	#nested list with all days in each year
	
	for file in files:
		x = re.findall(pattern1, file)
		#print(x)
		years.append(x[1])

	years = list(set(years))
	years.sort()
	year_files = {}	#list of all files in a year
	for i in range(len(years)):
		year_file = [file for file in files if file[-8:-6] == years[i]]
		year_files[years[i]] = year_file
	
	for i in year_files:
		year_days = []	#list of all days in a year
		for file in year_files[i]:
			x = re.findall(pattern, file)
			year_days.append(x)
		year_days = np.unique(np.asarray(year_days))
		#year_days.sort()
		all_year_days[i] = year_days

	#print(all_year_days)
		
	for year in all_year_days:
		days = all_year_days[year]
		for day in days:
			day_files = [file for file in year_files[year] if day in file]	#list of all files corresponding to a particular day
			list_day = []	#list of dataframes in a day
			for x in day_files:
				sat_data = np.loadtxt(x, delimiter = ',', usecols = [0,1,2,4,5,22,41])
				df = pd.DataFrame(sat_data, columns = titles)
				list_day.append(df)	
			#print(len(list_day))
			df_day = pd.concat(list_day)	#single dataframe for an entire day
			all_dfs["{}_{}".format(day, year)] = df_day	#master dataframe for the entire dataset	

	return all_dfs
	
	
def block_creator(path, block_size):
	"""Combines hourly information on TEC into blocks of information based on user preference
	
	Args:
		path (str): Full path of files 
		block_size (int): Block size (can be 10 or 100)
		
	Returns:
		dict: A dictionary whose key has two parts: the first and second parts represent the block and year respectively. Values are Pandas dataframes corresponding to that block and year.
	"""
	files = glob.glob(path)
	if block_size == 10:
		block = [file_name[-13:-11]+'_'+file_name[-8:-6] for file_name in files]
	if block_size == 100:
		block = [file_name[-13:-12]+'_'+file_name[-8:-6] for file_name in files]
	block = list(set(block))
	block.sort()
	#print(block)
	
	comb_dict = {}
	for days in block:
		day_comb, year = days.split('_')
		label = path.format(day_comb, year)
		#print(label)
		dfs = day(glob.glob(label))
		#print(dfs)
		dfs_list = []
		for st in dfs:
			dfs_list.append(dfs[st])
		block_df = pd.concat(dfs_list)
		comb_dict[days] = block_df
	return comb_dict
	

#files = glob.glob("*.ismr")
#block_list = block_creator(files)
#parent_dir = '/Data/rpriyadarshan/ismr/gcd_mean_el_plots'
#for directory in block_list:
#	path = os.path.join(parent_dir, directory)
#	os.mkdir(path)

		
path = "/Data/rpriyadarshan/ismr/ismr_files/PUNE3???.17_.ismr"
df = block_creator(path, block_size = 100)
print(df)




from setuptools import setup, find_packages

setup(
	name = "ionstruct",
	version = "0.0.1",
	author = "Rahul Priyadarshan R",
	author_email = "rahulpriyadarshan19@gmail.com",
	description = "A Python package to generate visualizations of TEC measurements in the ionosphere.",
	packages = find_packages()
	
)



from VTEC import VTEC_averaged, clean
from day import day
import numpy as np
import matplotlib.pyplot as plt
import glob
from scipy.interpolate import interp1d
from pickle import load
import time

#2018
all_dfs = day(glob.glob("*.18_.ismr"))
#all_dfs_18 = day(glob.glob("*.18_.ismr"))
#all_dfs = dict(all_dfs_17)
#all_dfs.update(all_dfs_18)
for day in all_dfs:
	all_dfs[day] = clean(all_dfs[day], GPS = True, elevation = True, TEC = True, locktime = True)
print("Cleaned")
VTEC_dict = VTEC_averaged(all_dfs, "map3")
#print(VTEC_dict['103_18'])

main_diff_array = []
main_VTEC_array = []
main_IRI_array = []
#diff_VTEC_dict = {}
for day in VTEC_dict:
	print(day)
	day_VTEC_dict = VTEC_dict[day]
	median_VTEC = day_VTEC_dict['{}_median_VTEC'.format(day)]
	print(len(median_VTEC))
	TOW = day_VTEC_dict['{}_TOW'.format(day)]
	TOW = TOW%86400
	#median_VTEC = median_VTEC[2:-2]
	iri_time_part = day_VTEC_dict['{}_IRI_time'.format(day)]
	iri_TEC_part = day_VTEC_dict['{}_IRI_TEC'.format(day)]
	#print(len(median_VTEC))
	print(iri_time_part)
	print(iri_TEC_part)
	if len(iri_time_part) > 1:
		f = interp1d(iri_time_part, iri_TEC_part)
		iri_time = np.arange(iri_time_part[0], iri_time_part[-1] + 60, 60)
		iri_TEC = f(iri_time)
		#print(iri_time[0] == iri_time_part[0])
		#print(iri_TEC[0] == iri_TEC_part[0])
		mask = np.isin(TOW[2:-2], iri_time)
		indices = np.where(mask == True)
		print(indices)
		#print(iri_time)
		#print(TOW)
		VTEC = np.empty(len(iri_time))
		VTEC[:] = np.nan
		print(median_VTEC[indices])
		VTEC[indices] = median_VTEC[indices]
		#VTEC[mask] = median_VTEC[mask]
		#print(VTEC[indices])
		diff = iri_TEC - VTEC
		print(diff)
		main_diff_array.append(diff)
		main_VTEC_array.append(VTEC)
		main_IRI_array.append(iri_TEC)
	else:
		iri_time = np.arange(iri_time_part[0], iri_time_part[-1] + 60, 60)
		print(len(iri_time))
		VTEC = np.empty(len(iri_time))
		VTEC[:] = np.nan
		print(VTEC)
		main_diff_array.append(VTEC)
		main_VTEC_array.append(VTEC)
		
img = plt.imshow(main_diff_array, cmap = 'seismic', aspect = 'auto')
plt.xlabel("Minute of the day")
plt.ylabel("Day of year 2018")
bar = plt.colorbar(img)
bar.set_label("VTEC(IRI) - VTEC(GPS) in TEC units")
plt.savefig('/Data/rpriyadarshan/ismr/yearly_trends_18_seismic.png')
plt.close()

img1 = plt.imshow(main_VTEC_array, aspect = "auto")
plt.xlabel("Minute of the day")
plt.ylabel("Day of year 2018")
bar1 = plt.colorbar(img1)
bar1.set_label("VTEC(GPS) in TEC units")
plt.savefig('/Data/rpriyadarshan/ismr/yearly_trends_VTEC_18_final.png')
plt.close()

img1 = plt.imshow(main_IRI_array, aspect = "auto")
plt.xlabel("Minute of the day")
plt.ylabel("Day of year 2018")
bar2 = plt.colorbar(img1)
bar2.set_label("VTEC(IRI) in TEC units")
plt.savefig('/Data/rpriyadarshan/ismr/yearly_trends_IRI_18_final.png')
plt.close()
#main_diff_array = np.asarray(main_diff_array)
#print(np.shape(main_diff_array))




"""Functions to provide visualizations of Vertical Total Electron Content (VTEC)"""
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import rc
import pandas as pd
import glob
from day import day, block_creator
import datetime
import mapping_functions as mapf
from itertools import combinations
from iri2016 import timeprofile

#rc("text", usetex = True)
R_earth = 6371
h_ion = 350
d = 100
lat_GMRT = 19.0919   #latitude of GMRT
long_GMRT = 74.0506  #longitude of GMRT

def TOW2UT(TOW):
	"""Time Of Week (TOW) to Universal Time (UT)
	
	Converts time in seconds measured since the start of the week to time of the day in Universal Time.
	
	Args:
		TOW (int): Time of Week in seconds
		
	Returns:
		str: Time in Universal Time Coordinates
	"""
	TOW = TOW%86400
	x = str(datetime.timedelta(seconds = TOW))
	return x
	
def UT2TOW(UT):
	"""Universal Time (UT) to Time Of Week (TOW)
	
	Converts time of the day in Universal Time to time in seconds measured since the start of the week.
	
	Args:
		UT (str): Universal Time
		
	Returns:
		int: Time Of Week (TOW)
	"""
	hrs, mins, sec = UT.split(':')
	hrs = int(hrs)
	mins = int(mins)
	sec = int(sec)
	TOW = hrs*3600 + mins*60 + sec
	return TOW

def rms(array):
	return np.sqrt(np.mean(array**2))

def VTEC(STEC, elevation, map_func):
	"""Vertical Total Electron Content (VTEC)
	
	Vertical Total Electron Content (VTEC) given a Slant Total Electron Content (STEC) value, angle of elevation and choice of mapping function.
	
	Args:
		STEC (float): Slant Total Electron Content measurement in TEC units
		elevation (float): Angle of elevation in degrees
		map_func (str): Choice of mapping function
		
	Returns:
		float: Vertical Total Electron Content measurement in TEC units
	"""
	map_method = getattr(mapf, map_func)
	VTEC = STEC/map_method(elevation)
	return VTEC

def clean(df, GPS = False, elevation = False, TEC = False, VTEC = False, locktime = False):
	"""Clean
	
	Removes erroneous values of VTEC
	
	Args:
		df (dataframe): Master dataframe containing TEC measurements
		GPS (bool): (default: False) If True, only GPS satellite data is included and vice-versa
		elevation (bool): (default: False) If True, only data for which elevation is greater than 30 degrees is included
		TEC (bool): (default: False) If True, only data with positive TEC values are included and vice-versa
		VTEC (bool): (default: False) If True, only data with positive VTEC values are included and vice-versa
		locktime (bool): (default: False) If True, only data with locktime greater than 3 minutes are included and vice-versa
	"""
	if elevation == True:
		df = df[df['elevation'] > 30]
	if TEC == True:
		df = df[df['TEC'] > 0]
	if VTEC == True:
		df = df[df['VTEC'] > 0]
	if locktime == True:
		df = df[df['locktime'] > 180]
	if GPS == True:
		df = df[(df['SVID'] >= 1) & (df['SVID'] <= 37)]
	return df	

def VTEC(STEC, elevation, map_func):
	"""Estimates VTEC from STEC provided an elevation and mapping function
	
	Args:
		STEC (float): Slant Total Electron Content in TEC units
		elevation (float): Angle of elevation
		map_func (str): String describing choice of mapping function
		
	Returns:
		float: Vertical Total Electron Content in TEC units
	
	"""
	map_method = getattr(mapf, map_func)
	VTEC = STEC/map_method(elevation)
	return VTEC

def great_circle_distance(el1, el2, az1, az2):
	"""Great Circle Distance
	
	Returns the great circle distance between two points
	
	Args:
		el1 (float): Elevation of 1st point
		el2 (float): Elevation of 2nd point
		az1 (float): Azimuth of 1st point
		az2 (float): Azimuth of 2nd point
		
	Returns:
		float: Great circle distance in degrees
	"""
	el1 = np.deg2rad(el1)
	el2 = np.deg2rad(el2)
	az1 = np.deg2rad(az1)
	az2 = np.deg2rad(az2)
	delta_el = el1 - el2
	delta_az = az1 - az2
	gcd = 2*np.arcsin(np.sqrt((np.sin(delta_el/2))**2 + np.cos(el1)*np.cos(el2)*(np.sin(delta_az/2))**2))
	return gcd

def VTEC_time(all_dfs, map_fn):
	"""Displays scatter plot(s) showing variation of VTEC with time for a set of day(s)
	
	Args:
		all_dfs (dict): Master dataframe containing TEC information for the given set of days. Obtained by running day.day()
		map_fn (str): String describing choice of mapping function
		  
	"""
	
	for day in all_dfs:
		print(day)
		year = day[4:]
		df_day = all_dfs[day]
		TOW = np.unique(df_day['TOW'])
		SVID = np.unique(df_day['SVID'])
		el = np.unique(df_day['elevation'])
		
		l = 0
		for satellite in SVID:
			df_sat = df_day.loc[df_day['SVID'] == satellite, ['TOW', 'elevation', 'TEC', 'locktime']]
			#df_p = df_sat.loc[(df_sat['TEC']>0) & (df_sat['elevation']>30), ['TOW', 'elevation', 'TEC', 'locktime']] 
			df_p = clean(df_sat, elevation = True, TEC = True)
			
			minlock = df_p.loc[df_p['locktime']<180, ['TOW', 'TEC', 'elevation', 'locktime']]
			minlock['VTEC'] = VTEC(minlock['TEC'], minlock['elevation'], map_func = map_fn)
			minlock_VTEC = clean(minlock, VTEC = True)
			maxlock = df_p.loc[df_p['locktime']>180, ['TOW', 'TEC', 'elevation', 'locktime']]
			maxlock['VTEC'] = VTEC(maxlock['TEC'], maxlock['elevation'], map_func = map_fn)
			maxlock_VTEC = clean(maxlock, VTEC = True)
			
			plt.figure(0)
			plt.scatter(minlock_VTEC['TOW'], minlock_VTEC['VTEC'], marker = 's', c = minlock_VTEC['elevation'], label = "Locktime $\leq$ 3 min" if l == 0 else "")
			plt.scatter(maxlock_VTEC['TOW'], maxlock_VTEC['VTEC'], marker = 'o', s = 3, c = maxlock_VTEC['elevation'], label = "Locktime $>$ 3 min" if l == 0 else "")
			l += 1
			
		h = (np.max(TOW) - np.min(TOW))/5
		t0 = np.min(TOW)
		t1 = np.min(TOW) + h
		t2 = np.min(TOW) + 2*h
		t3 = np.min(TOW) + 3*h
		t4 = np.min(TOW) + 4*h
		t5 = np.max(TOW)
		
		plt.figure(0)
		x = plt.clim(30,90)
		plt.colorbar()
		plt.xlabel("Time (UT)")
		plt.xticks([t0, t1, t2, t3, t4, t5], [TOW2UT(t0), TOW2UT(t1), TOW2UT(t2), TOW2UT(t3), TOW2UT(t4), TOW2UT(t5)])
		plt.ylabel("Vertical TEC (TECU)")
		d, y = day.split("_")
		plt.title("Vertical TEC ({}-{})".format(d, y))
		plt.legend()
		plt.grid()
		plt.show()
		#plt.savefig("/Data/rpriyadarshan/ismr/sat_TEC_plots/{}/{}_VTEC_{}.png".format(day, map_fn, day))
		#print("Saved")
		#plt.close()
		
	
def VTEC_STEC(all_dfs, map_fn):
	"""Displays scatter plot(s) showing variation of VTEC with STEC for a set of day(s)
	
	Args:
		all_dfs (dict): Master dataframe containing TEC information for the given set of days. Obtained by running day.day()
		map_fn (str): String describing choice of mapping function
		
	"""

	for day in all_dfs:
		print(day)
		year = day[4:]
		df_day = all_dfs[day]
		TOW = np.unique(df_day['TOW'])
		SVID = np.unique(df_day['SVID'])
		el = np.unique(df_day['elevation'])
		
		l = 0
		for satellite in SVID:
			df_sat = df_day.loc[df_day['SVID'] == satellite, ['TOW', 'elevation', 'TEC', 'locktime']]
			#df_p = df_sat.loc[(df_sat['TEC']>0) & (df_sat['elevation']>30), ['TOW', 'elevation', 'TEC', 'locktime']] 
			df_p = clean(df_sat, elevation = True, TEC = True)

			minlock = df_p.loc[df_p['locktime']<180, ['TOW', 'TEC', 'elevation']]
			minlock['VTEC'] = VTEC(minlock['TEC'], minlock['elevation'], map_func = map_fn)
			minlock_VTEC = clean(minlock, VTEC = True)
			maxlock = df_p.loc[df_p['locktime']>180, ['TOW', 'TEC', 'elevation']]
			maxlock['VTEC'] = VTEC(maxlock['TEC'], maxlock['elevation'], map_func = map_fn)
			maxlock_VTEC = clean(maxlock, VTEC = True)

			plt.figure(1)
			plt.scatter(minlock_VTEC['TEC'], minlock_VTEC['VTEC'], marker = 's', c = minlock_VTEC['elevation'], label = "Locktime $\leq$ 3 min" if l == 0 else "")
			plt.scatter(maxlock_VTEC['TEC'], maxlock_VTEC['VTEC'], marker = 'o', s = 3, c = maxlock_VTEC['elevation'], label = "Locktime $>$ 3 min" if l == 0 else "") 
			l += 1
	
		plt.figure(1)
		x = plt.clim(30,90)
		plt.colorbar()
		plt.xlabel("Slant TEC (TECU)")
		plt.ylabel("Vertical TEC (TECU)")
		d, y = day.split("_")
		plt.title("VTEC vs STEC ({}-{})".format(d, y))
		plt.legend()
		plt.grid()
		plt.show()
		#plt.savefig("/Data/rpriyadarshan/ismr/sat_TEC_plots/{}/{}_VTEC_STEC_{}.png".format(day, map_fn, day))
		#print("Saved")
		#plt.close()
	
	
def VTEC_averaged(all_dfs, map_fn):	
	"""Displays plot(s) showing variation of mean, median and RMS VTEC with time for a set of day(s)
	
	Args:
		all_dfs (dict): Master dataframe containing TEC information for the given set of days. Obtained by running day.day()
		map_fn (str): String describing choice of mapping function
		
	"""
	for day in all_dfs:
		print(day)
		year = day[4:]
		df_day = all_dfs[day]
		TOW = np.unique(df_day['TOW'])
		SVID = np.unique(df_day['SVID'])
		el = np.unique(df_day['elevation'])
		
		start_t = TOW[2]
		end_t = TOW[-2]
		iri_time = np.arange(start_t, end_t, 1*60)
		start_time = TOW2UT(start_t)
		end_time = TOW2UT(end_t)
		dayOfYear, year = day.split("_")
		Year = '20' + year
		d = datetime.datetime.strptime('{} {}'.format(dayOfYear, Year),'%j %Y')
		calendar_day = d.strftime('%Y-%m-%d')
		start = calendar_day + "T{}".format(start_time)
		end = calendar_day + "T{}".format(end_time)
		print(start_time)
		print(end_time)
		time_prof = timeprofile((start, end), datetime.timedelta(minutes = 1), [0, 2000, 10], lat_GMRT, long_GMRT)
		iri_TEC = time_prof.TEC/10**16
		plt.figure()
		plt.plot(iri_time, iri_TEC, '--', c = 'g', label = 'IRI')

		mean_VTEC = np.array([])
		median_VTEC = np.array([])
		RMS_VTEC = np.array([])
		for time in TOW:
			
			df_sat = df_day.loc[df_day['TOW'] == time, ['TOW', 'elevation', 'TEC', 'locktime']]
			#maxlock = df_p.loc[df_p['locktime']>180, ['TOW', 'TEC', 'elevation']]
			maxlock = clean(df_sat, elevation = True, TEC = True, locktime = True)
			#maxlock['VTEC'] = maxlock['TEC']/map_method(maxlock['elevation'])
			maxlock['VTEC'] = VTEC(maxlock['TEC'], maxlock['elevation'], map_func = map_fn)
			#maxlock_VTEC = maxlock.loc[maxlock['VTEC']>0, ['TEC', 'VTEC', 'TOW', 'elevation']]
			maxlock_VTEC = clean(maxlock, VTEC = True)

			m_VTEC = np.mean(maxlock_VTEC['VTEC'])
			mean_VTEC = np.append(mean_VTEC, m_VTEC)
			med_VTEC = np.median(maxlock_VTEC['VTEC'])
			median_VTEC = np.append(median_VTEC, med_VTEC)
			R_VTEC = rms(maxlock_VTEC['VTEC'])
			RMS_VTEC = np.append(RMS_VTEC, R_VTEC)
	
		plt.plot(TOW, mean_VTEC, c = 'blue', label = "$VTEC_{mean}$")
		plt.plot(TOW, median_VTEC, c = 'red', label = "$VTEC_{median}$")
		plt.plot(TOW, RMS_VTEC, c = 'black', label = "$VTEC_{RMS}$")
		h = (np.max(TOW) - np.min(TOW))/5
		t0 = np.min(TOW)
		t1 = np.min(TOW) + h
		t2 = np.min(TOW) + 2*h
		t3 = np.min(TOW) + 3*h
		t4 = np.min(TOW) + 4*h
		t5 = np.max(TOW)
		plt.xlabel("Time (UT)")
		plt.xticks([t0, t1, t2, t3, t4, t5], [TOW2UT(t0), TOW2UT(t1), TOW2UT(t2), TOW2UT(t3), TOW2UT(t4), TOW2UT(t5)])
		plt.ylabel("Vertical TEC (TECU)")
		d, y = day.split("_")
		plt.title("Vertical TEC ({}-{})".format(d, y))
		plt.legend()
		plt.grid()
		plt.show()
		#plt.savefig("/Data/rpriyadarshan/ismr/sat_TEC_plots/{}/IRI_GPS_only_{}_VTEC_averaged_{}.png".format(day, map_fn, day))
		#print("Saved")
		#plt.close()
		

def VTEC_comparison(all_dfs, map_fn, print_output = False):

	for day in all_dfs:
		year = day[4:]
		df_day = all_dfs[day]
		
		SVID = np.unique(df_day['SVID'])

		#df = df_day.loc[(df_day['elevation'] > 30) & (df_day['elevation'] < 35) & (df_day['locktime'] > 180) & (df_day['TOW'] == 600), ['SVID', 'TOW', 'TEC', 'elevation']]
		#df = df_day.loc[(df_day['locktime'] > 180) & (df_day['elevation'] > 30) & (df_day['TEC'] > 0), ['SVID', 'TOW', 'TEC', 'elevation']]
		df = clean(df_day, elevation = True, TEC = True, locktime = True)
		el = np.unique(df['elevation'])
		el = el[~np.isnan(el)]
		df['VTEC'] = VTEC(df['TEC'], df['elevation'], map_func = map_fn)
		bin_angles = np.arange(np.min(el), np.max(el) + 2, 2)
		bin_labels = ['{}-{}'.format(bin_angles[i], bin_angles[i+1]) for i in range(len(bin_angles)-1)]
		df['elevation_bins'] = pd.cut(df['elevation'], bins = bin_angles, labels = bin_labels)
		
		delta_VTEC_info = {}
		for bin in bin_labels:
			#print(bin)
			df_bin = df.loc[df['elevation_bins'] == bin, ['SVID', 'TOW', 'TEC', 'VTEC', 'elevation', 'azimuth']]
			#print("Bin size: {}".format(df_bin.size))
			TOW = np.asarray(df_bin['TOW'])

			idx_sort = np.argsort(TOW)
			sorted_TOW = TOW[idx_sort]
			vals, idx_start, count = np.unique(sorted_TOW, return_counts = True, return_index = True)
			res = np.split(idx_sort, idx_start[1:])
			vals = vals[count > 1]
			res = filter(lambda x: x.size > 1, res)
			list_indices = list(res)
			#print(list_indices)

			mean_els = []
			delta_VTECs = []
			delta_azs = []
			gcds = []
			SVIDs = []
			for indices in list_indices:
				comb = list(combinations(indices, 2))
				#print(comb)
				for index in comb:
					a,b = index
					el1 = df_bin['elevation'].iloc[a]
					el2 = df_bin['elevation'].iloc[b]
					mean_el = (el1 + el2)/2
					mean_els.append(mean_el)
					az1 = df_bin['azimuth'].iloc[a]
					az2 = df_bin['azimuth'].iloc[b]
					delta_az = abs(az1 - az2)
					delta_azs.append(delta_az)
					VTEC1 = df_bin['VTEC'].iloc[a]
					VTEC2 = df_bin['VTEC'].iloc[b]
					delta_VTEC = abs(VTEC1 - VTEC2)
					delta_VTECs.append(delta_VTEC)
					gcd = great_circle_distance(el1, el2, az1, az2)
					gcd = np.rad2deg(gcd)
					gcds.append(gcd)
					SVID1 = df_bin['SVID'].iloc[a]
					SVID2 = df_bin['SVID'].iloc[b]
					SVID_pair = '{}-{}'.format(SVID1, SVID2)
					SVIDs.append(SVID_pair)
					#print("Mean elevation: {}".format(mean_el))
					#print("Difference in azimuth: {}".format(delta_az))
					#print("Difference in VTEC: {}".format(delta_VTEC))
					#print("Great circle distance: {}".format(gcd))

			delta_VTECs = np.asarray(delta_VTECs)
			mean_els = np.asarray(mean_els)
			delta_azs = np.asarray(delta_azs)
			gcds = np.asarray(gcds)
			data_dict = {'Mean elevation': mean_els, 'Azimuth difference': delta_azs, 'VTEC difference': delta_VTECs, 'Great circle distance': gcds, 'Satellite ids': SVIDs}
			df_el_bin = pd.DataFrame(data_dict)
			delta_VTEC_info[bin] = df_el_bin
			#delta_VTEC_info['{}_delta_VTECs'.format(bin)] = delta_VTECs
			#delta_VTEC_info['{}_mean_els'.format(bin)] = mean_els

		np.save("/Data/rpriyadarshan/ismr/sat_TEC_plots/{}/{}_delta_VTEC_info.npy".format(day, day), delta_VTEC_info)
		#print(delta_VTEC_info)
		print(day + "Saved!")



def VTEC_min_comparison(all_dfs, map_fn):
	"""Displays plot(s) showing great circle distances between pairs of satellites seen in similar elevation bins for a block of day(s)
	
	Args:
		all_dfs (dict): Master dataframe containing TEC information for the given set of days. Obtained by running day.day()
		map_fn (str): String describing choice of mapping function
	"""
	
	main_df_titles = ['elevations', 'mean_elevation', 'VTECs', 'STECs', 'delta_VTEC', 'great_circle_distance', 'day', 'time', 'SVIDs', 'azimuthal_angles'] 
	main_dict = {}
	
	for day in all_dfs:
	
		day_block, year = day.split('_')
		els = []
		VTECs = []
		STECs = []
		delta_VTECs = []
		gcds = []
		days = []
		TOWs = []
		SVIDs = []
		azs = []
		mean_els = []
	
		#year = day[4:]
		df_day = all_dfs[day]
		SVID = np.unique(df_day['SVID'])
		TOW = np.unique(df_day['TOW'])
		
		df = clean(df_day, elevation = True, TEC = True, locktime = True)
		el = np.unique(df['elevation'])
		el = el[~np.isnan(el)]
		df['VTEC'] = VTEC(df['TEC'], df['elevation'], map_func = map_fn)
		bin_angles = np.arange(np.min(el), np.max(el) + 2, 2)
		bin_labels = ['{}-{}'.format(bin_angles[i], bin_angles[i+1]) for i in range(len(bin_angles)-1)]
		df['elevation_bins'] = pd.cut(df['elevation'], bins = bin_angles, labels = bin_labels)
		
		for time in TOW:
			df_time = df.loc[df['TOW'] == time, ['SVID', 'TOW', 'TEC', 'VTEC', 'elevation', 'azimuth', 'elevation_bins']]
			el = np.asarray(df_time['elevation'])
			
			idx_sort = np.argsort(el)
			sorted_el = el[idx_sort]
			vals, idx_start, count = np.unique(sorted_el, return_counts = True, return_index = True)
			res = np.split(idx_sort, idx_start[1:])
			vals = vals[count > 1]
			res = filter(lambda x: x.size > 1, res)
			list_indices = list(res)
			#print(list_indices)
						
			for indices in list_indices:
				comb = list(combinations(indices, 2))
				#print(comb)
				for index in comb:
					a,b = index
					el1 = df_time['elevation'].iloc[a]
					el2 = df_time['elevation'].iloc[b]
					el_array = np.array([el1, el2])
					els.append(el_array)
					mean_el = (el1 + el2)/2
					mean_els.append(mean_el)
					
					az1 = df_time['azimuth'].iloc[a]
					az2 = df_time['azimuth'].iloc[b]
					delta_az = abs(az1 - az2)
					az_array = np.array([az1, az2])
					azs.append(az_array)
					#delta_azs.append(delta_az)
					
					STEC1 = df_time['TEC'].iloc[a]
					STEC2 = df_time['TEC'].iloc[b]
					STEC_array = np.array([STEC1, STEC2])
					STECs.append(STEC_array)
					
					VTEC1 = df_time['VTEC'].iloc[a]
					VTEC2 = df_time['VTEC'].iloc[b]
					VTEC_array = np.array([VTEC1, VTEC2])
					VTECs.append(VTEC_array)
					delta_VTEC = (VTEC1 - VTEC2)
					delta_VTECs.append(delta_VTEC)
					
					gcd = great_circle_distance(el1, el2, az1, az2)
					gcd = np.rad2deg(gcd)
					gcds.append(gcd)
					
					SVID1 = df_time['SVID'].iloc[a]
					SVID2 = df_time['SVID'].iloc[b]
					SVID_array = [SVID1, SVID2]
					SVIDs.append(SVID_array)
					
					TOW_converted = TOW2UT(time)
					TOWs.append(TOW_converted)
					
					days.append(day)
		
	
		main_list = [els, mean_els, VTECs, STECs, delta_VTECs, gcds, days, TOWs, SVIDs, azs]
		for i in range(len(main_list)):
			main_dict[main_df_titles[i]] = main_list[i]
			#print(len(main_list[i]))
		df_main = pd.DataFrame(main_dict)
		SVID = np.unique(df['SVID'])
		#print(df_main[['elevations', 'azimuthal_angles', 'STECs', 'time', 'SVIDs']].head(20))
		#print(SVID)
		
		for no in SVID:
			plt.figure()
			df_main['SVID_presence'] = [True if no in SVID_list else False for SVID_list in df_main['SVIDs']]
			df_SVID = df_main.loc[df_main['SVID_presence'] == True]
			for i in range(len(df_SVID)):
				SVID_list = df_SVID['SVIDs'].iloc[i]
				if no == SVID_list[1]:
					df_SVID['delta_VTEC'].iloc[i] *= -1		
			plt.scatter(df_SVID['mean_elevation'], df_SVID['great_circle_distance'], c = df_SVID['delta_VTEC'], cmap = 'seismic', alpha = 0.4)
			x = plt.clim(-15,15)
			plt.colorbar()
			plt.xlabel("Mean elevation")
			plt.ylabel("Great circle distance")
			plt.title("{}, year {}, SVID: {}".format(day_block, year, no))
			plt.show()
			#plt.savefig("/Data/rpriyadarshan/ismr/gcd_mean_el_plots/{}/GPS_only_no_abs_{}_SVID_{}.png".format(day, day, no))
			#plt.close()
		print("{} done!".format(day))

'''
filestring = "PUNE1???.18_.ismr"
all_dfs = block_creator(glob.glob(filestring), block_size = 100)
for st in all_dfs:
	all_dfs[st] = clean(all_dfs[st], GPS = True)
VTEC_min_comparison(all_dfs, map_fn = "map3")
print("done!")
'''

'''
all_dfs = day(glob.glob("*.ismr"))
for day in all_dfs:
	all_dfs[day] = clean(all_dfs[day], GPS = True)
print("Cleaned")
VTEC_averaged(all_dfs, map_fn = "map3")
#print(UT2TOW('23:59:41'))
'''
'''
for st in all_dfs:
	all_dfs[st] = clean(all_dfs[st], GPS = True)
VTEC_time(all_dfs, map_fn = "map3")
'''
#df = day(glob.glob("PUNE323?.17_.ismr"))
#VTEC_time(df, map_fn = "map3")
